package designPatterns;

import java.util.*;

public class CompositeClient {
    public static void main(String[] args) {
        GraphicalComponent circ1, circ2, sq1, sq2, comp1, comp2;
        circ1 = new Circle(5,2);
        circ2 = new Circle(3,4);
        sq1 = new Square(5,2);
        sq2 = new Square(7,3);
        comp1 = new CompositeComponent();
        comp2 = new CompositeComponent();
        comp1.add(circ1);
        comp1.add(circ2);
        comp2.add(sq1);
        comp2.add(comp1);
        System.out.println("Borders: " + comp2.leftBorder() + " - " + comp2.rightBorder());
        comp2.moveX(2); // uniform interface for client
        sq1.moveX(3);
        System.out.println("Borders: " + comp2.leftBorder() + " - " + comp2.rightBorder());
        sq2.add(sq1); // but no compile-time checks
    }
}

class EmptyIterator<T> implements Iterator<T> {

    public boolean hasNext() {
        return false;
    }
    
    public T next() {
        throw new NoSuchElementException();
    }

    public void remove() {
        throw new UnsupportedOperationException();
    }
    
}

abstract class GraphicalComponent {
    private final static Iterator<GraphicalComponent> EMPTY_ITERATOR = new EmptyIterator<>();
    
    // default implementation for leafs
    public void add(GraphicalComponent c) {
        throw new RuntimeException("add not possible for leafs");
    }
    public abstract void moveX(double diffx);
    public Iterator<GraphicalComponent> children() {
        return EMPTY_ITERATOR;
    }
    public abstract double leftBorder();
    public abstract double rightBorder();
    public double width() {
        return this.rightBorder() - this.leftBorder();
    }
}

class Circle extends GraphicalComponent {
    private double x;
    private double r;
    
    public Circle(double x, double r) {
        this.x = x;
        this.r = r;
    }
    
    public double leftBorder() {
	return this.x - this.r;
    }
    
    public double rightBorder() {
	return this.x + this.r;
    }
    
    public void moveX(double diffx) {
        this.x += diffx;
    }
}

class Square extends GraphicalComponent {
    
    public Square(double left, double width) {
        this.left = left;
        this.width = width;
    }
    
    private double left;
    private double width;
    
    public double leftBorder() {
	return this.left;
    }
    
    public double rightBorder() {
	return this.left + this.width;
    }
    
    public void moveX(double diffx) {
        this.left += diffx;
    }

}

class CompositeComponent extends GraphicalComponent implements
	Iterable<GraphicalComponent> {
    private Set<GraphicalComponent> children = new HashSet<>();
    
    public void add(GraphicalComponent c) {
        this.children.add(c);        
    }
    
    public Iterator<GraphicalComponent> children() {
        return this.children.iterator();
    }
    
    public Iterator<GraphicalComponent> iterator() {
        return this.children();
    }
    
    public void moveX(double diffx) {
        for (GraphicalComponent comp : this) {
            comp.moveX(diffx);
        }
    }
    
    public double leftBorder() {
        double border = Double.MAX_VALUE;
        for (GraphicalComponent comp : this) {
            border = Math.min(border, comp.leftBorder());
        }
        return border;
    }
    
    public double rightBorder() {
        double border = Double.MIN_VALUE;
        for (GraphicalComponent comp : this) {
            border = Math.max(border, comp.rightBorder());
        }
        return border;
    }
}

