package designPatterns;

import java.util.*;

public class ObserverClient {
    public static void main(String[] args) throws InterruptedException {
        final Clock c = new Clock();
        // register two completely independent observer
        c.addObserver(new Observer() {
            public void update(Observable o, Object message) {
                System.out.println("Clock is running: " + c.getStatus());
            }            
        });
        c.addObserver(new Observer() {
            public void update(Observable o, Object message) {
                System.out.println("Elapsed time: " + c.getTime());
            }            
        });
        
        Thread.sleep(500);
        c.start();
        Thread.sleep(4000);
        c.stop();
        Thread.sleep(2000);
        c.start();
        Thread.sleep(250);
        
        // output shows that there are things to improve:
        // 1. Status observer also prints output if there is change in time
        // 2. Clock fires setChanged(), although nothing changed w.r.t. observable 
        //    values getStatus() and getTime()
    }
}

class Quartz extends Observable implements Runnable {
    private static final Quartz quartz = new Quartz();
    private Quartz() {
        Thread t = new Thread(this);
        t.setDaemon(true); // if this is only thread, then stop program
        t.start();
    }
    public static Quartz getInstance() {
        return quartz; // there is only 1 quartz, singleton pattern
    }
    public void run()  {
        while (true) {
            try {
                Thread.sleep(50); // tick approx. every 50 milliseconds
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
            this.setChanged();
            this.notifyObservers();
        }
    }
}

class Clock extends Observable implements Observer {        
    
    // volatile and synchronized are used because of concurrency:
    // both the Quartz-Thread and the User-Thread will access clocks
    private volatile long lastUpdate; 
    private volatile long elapsed = 0;
    private volatile boolean running = false;
    
    public Clock() {
        Quartz.getInstance().addObserver(this);
    }
    
    public boolean getStatus() {
        return this.running;
    }
    
    public String getTime() {
        long time = this.elapsed;
        long ms = time % 1000;
        time /= 1000;
        long s = time % 60;
        time /= 60;
        long min = time % 60;
        return min + ":" + (s < 10 ? "0" + s : s) + "," + (ms / 100);
    }

    public void update(Observable src, Object message) {
        this.update();
        this.notifyObservers();
    }

    private synchronized void update() {        
	if (this.running) {
            long newTime = System.currentTimeMillis();
	    this.elapsed += (newTime - this.lastUpdate);
	    this.lastUpdate = newTime;
            this.setChanged();
        }
    }
    
    public synchronized void stop() {
	if (this.running) {
            update();
	    this.running = false;
            this.setChanged();
        }
        this.notifyObservers();
    }

    public synchronized void start() {
	if (!this.running) {
	    this.lastUpdate = System.currentTimeMillis();
	    this.running = true;
            this.setChanged();
        }
        this.notifyObservers();
    }
}
